/* Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CCUnit Library.

   The CCUnit Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   The CCUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CCUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

/*
 * $Id: CCUnitTestFixture.c,v 1.8 2003/10/01 20:50:31 tsutsumi Exp $
 */

/**
 * @file
 * TestFixture module implementation.
 */

#include <ccunit/CCUnitTestFixture.h>
#include <ccunit/CCUnitTestResult.h>
#include <assert.h>
#include <setjmp.h>

/**
 * run test fixtures exception.
 */
jmp_buf _ccunit_run_env;

/**
 * run tests exception.
 */
jmp_buf _ccunit_runTest_env;

extern void _ccunit_startTest (CCUnitTestResult* result,
			       struct CCUnitTestCase* test);
extern void _ccunit_endTest (CCUnitTestResult* result,
			     struct CCUnitTestCase* test);

inline void ccunit_addTestCase (CCUnitTestFixture* fixture,
				CCUnitTestCase* testCase)
{
  ccunit_addList (&fixture->testCases, testCase);
}

CCUnitTestCase* ccunit_addNewTestCase (CCUnitTestFixture* fixture,
				       const char* name,
				       const char* desc,
				       void (*runTest)())
{
  CCUnitTestCase* c;
  if (!fixture)
    return NULL;
  c = ccunit_newTestCase (name, desc, runTest);
  if (!c)
    return NULL;
  ccunit_addTestCase (fixture, c);
  return c;
}

/*
 * Runs the bare test sequence.
 *
 * @throws _ccunit_run_env
 */
static void runTest (CCUnitTestCase* testCase,
		     CCUnitTestFunc* setUp,
		     CCUnitTestFunc* tearDown,
		     CCUnitTestResult* result)
{
  int failure = 0;
  int runFailure = 0;
  if (setUp && setUp->runTest)
    {
      if ((failure = setjmp (_ccunit_runTest_env)) == 0)
	setUp->runTest ();
      else
	{
	  ((CCUnitTestFailure*)failure)->testCase = setUp;
	  longjmp (_ccunit_run_env, failure);
	}
    }
  if ((runFailure = setjmp (_ccunit_runTest_env)) == 0)
    {
      result->runCount ++;
      testCase->runTest ();
    }
  else
    ((CCUnitTestFailure*)runFailure)->testCase = testCase;
  if (tearDown && tearDown->runTest)
    {
      if ((failure = setjmp (_ccunit_runTest_env)) == 0)
	tearDown->runTest ();
      else
	{
	  ((CCUnitTestFailure*)failure)->testCase = tearDown;
	  ccunit_addFailure (result, (CCUnitTestFailure*)failure);
	}
    }
  if (runFailure)
    longjmp (_ccunit_run_env, runFailure);
}

/**
 * Runs the test fixture and collects the results in CCUnitTestResult.
 * @param test A test to run.
 * @param result A result container.
 */
static void run (CCUnitTest* test, CCUnitTestResult* result)
{
  CCUnitTestFixture* fixture = (CCUnitTestFixture*)test;
  int failure = 0;
  CCUnitListIterator itor;
  CCUnitTestCase* testCase;
  assert (test->type == ccunitTypeFixture);
  ccunit_initListIterator (&fixture->testCases, &itor);
  while ((testCase = ccunit_nextListIterator (&itor)) != NULL)
    {
      _ccunit_startTest (result, testCase);
      if ((failure = setjmp (_ccunit_run_env)) == 0)
	runTest (testCase, fixture->setUp, fixture->tearDown, result);
      else
	ccunit_addFailure (result, (CCUnitTestFailure*)failure);
      _ccunit_endTest (result, testCase);
    }
}

/**
 * Destruct test fixture.
 * @param test destruct test.
 */
static void destroy (CCUnitTest* test)
{
  CCUnitTestFixture* fixture;
  assert (test->type == ccunitTypeFixture);
  fixture = (CCUnitTestFixture*)test;
  safe_free ((char*)fixture->name);
  ccunit_deleteTestFunc (fixture->setUp);
  ccunit_deleteTestFunc (fixture->tearDown);
  ccunit_deleteList (&fixture->testCases,
		     (void (*)(void*))ccunit_deleteTestCase);
}

CCUnitTestFixture* ccunit_newTestFixture (const char* name,
					  CCUnitTestFunc* setUp,
					  CCUnitTestFunc* tearDown)
{
  CCUnitTestFixture* fixture = calloc (1, sizeof (*fixture));
  ccunit_initTest (&fixture->test, ccunitTypeFixture, run, destroy);
  fixture->name = safe_strdup (name);
  ccunit_initList (&fixture->testCases);
  fixture->setUp = setUp;
  fixture->tearDown = tearDown;
  return fixture;
}

inline struct CCUnitTestResult* ccunit_runTestFixture (CCUnitTestFixture* f)
{
  CCUnitTestResult* result;
  if (!f)
    return NULL;
  result = ccunit_newTestResult ();
  if (!result)
    return NULL;
  f->test.run (&f->test, result);
  return result;
}
